@@ -1,7 +1,7 @@
 /* common.c - Functions that are common to server and clinet
  * Rob Siemborski
  * Tim Martin
- * $Id: common.c,v 1.104 2004/09/22 21:35:34 shadow Exp $
+ * $Id: common.c,v 1.109 2005/03/01 22:28:30 shadow Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -166,6 +166,7 @@ int _sasl_add_string(char **out, size_t *alloclen,
 void sasl_version(const char **implementation, int *version) 
 {
     if(implementation) *implementation = implementation_string;
+    /* NB: the format is not the same as in SASL_VERSION_FULL */
     if(version) *version = (SASL_VERSION_MAJOR << 24) | 
 		           (SASL_VERSION_MINOR << 16) |
 		           (SASL_VERSION_STEP);
@@ -591,14 +592,14 @@ int sasl_getprop(sasl_conn_t *conn, int propnum, const void **pvalue)
 	      break;
 	  }
 	  *((const char **)pvalue) =
-	      ((sasl_client_conn_t *)conn)->mech->plugname;
+	      ((sasl_client_conn_t *)conn)->mech->m.plugname;
       } else if (conn->type == SASL_CONN_SERVER) {
 	  if(!((sasl_server_conn_t *)conn)->mech) {
 	      result = SASL_NOTDONE;
 	      break;
 	  }
 	  *((const char **)pvalue) =
-	      ((sasl_server_conn_t *)conn)->mech->plugname;
+	      ((sasl_server_conn_t *)conn)->mech->m.plugname;
       } else {
 	  result = SASL_BADPARAM;
       }
@@ -610,14 +611,14 @@ int sasl_getprop(sasl_conn_t *conn, int propnum, const void **pvalue)
 	      break;
 	  }
 	  *((const char **)pvalue) =
-	      ((sasl_client_conn_t *)conn)->mech->plug->mech_name;
+	      ((sasl_client_conn_t *)conn)->mech->m.plug->mech_name;
       } else if (conn->type == SASL_CONN_SERVER) {
 	  if(!((sasl_server_conn_t *)conn)->mech) {
 	      result = SASL_NOTDONE;
 	      break;
 	  }
 	  *((const char **)pvalue) =
-	      ((sasl_server_conn_t *)conn)->mech->plug->mech_name;
+	      ((sasl_server_conn_t *)conn)->mech->m.plug->mech_name;
       } else {
 	  result = SASL_BADPARAM;
       }
@@ -1025,11 +1026,20 @@ _sasl_conn_getopt(void *context,
 
 #ifdef HAVE_SYSLOG
 /* this is the default logging */
-static int _sasl_syslog(void *context __attribute__((unused)),
+static int _sasl_syslog(void *context,
 			int priority,
 			const char *message)
 {
     int syslog_priority;
+    sasl_server_conn_t *sconn;
+
+    if (context) {
+	if (((sasl_conn_t *)context)->type == SASL_CONN_SERVER) {
+	    sconn = (sasl_server_conn_t *)context;
+	    if (sconn->sparams->log_level < priority) 
+		return SASL_OK;
+	}
+    }
 
     /* set syslog priority */
     switch(priority) {
@@ -1838,6 +1848,8 @@ _sasl_getpath(void *context __attribute__((unused)),
   if (! path)
     return SASL_BADPARAM;
 
+  *path = NULL;
+
   /* Honor external variable only in a safe environment */
   if (getuid() == geteuid() && getgid() == getegid())
     *path = getenv(SASL_PATH_ENV_VAR);
